// eslint-disable-next-line no-unused-vars
import { EVENT_CROP, EVENT_CROP_END, EVENT_CROP_MOVE, EVENT_CROP_START, EVENT_DBLCLICK, EVENT_POINTER_DOWN, EVENT_POINTER_MOVE, EVENT_POINTER_UP, EVENT_RESIZE, EVENT_WHEEL, EVENT_ZOOM } from './constants'
// eslint-disable-next-line no-unused-vars
import { addListener, isFunction, removeListener } from './utilities'

export default {
  methods: {
    bind() {
      const { element, $props, cropper } = this

      addListener(element, EVENT_CROP_START, this.cropstart)

      addListener(element, EVENT_CROP_MOVE, this.cropmove)

      addListener(element, EVENT_CROP_END, this.cropend)

      addListener(element, EVENT_CROP, this.cropping)

      addListener(element, EVENT_ZOOM, this.zooming)

      addListener(cropper, EVENT_POINTER_DOWN, (this.onCropStart = this.cropStart.bind(this)))

      if ($props.zoomable && $props.zoomOnWheel) {
        addListener(cropper, EVENT_WHEEL, (this.onWheel = this.wheel.bind(this)), {
          passive: false,
          capture: true
        })
      }

      if ($props.toggleDragModeOnDblclick) {
        addListener(cropper, EVENT_DBLCLICK, (this.onDblclick = this.dblclick.bind(this)))
      }

      addListener(
        element.ownerDocument,
        EVENT_POINTER_MOVE,
        (this.onCropMove = this.cropMove.bind(this))
      )
      addListener(
        element.ownerDocument,
        EVENT_POINTER_UP,
        (this.onCropEnd = this.cropEnd.bind(this))
      )

      if ($props.responsive) {
        addListener(window, EVENT_RESIZE, (this.onResize = this.resize.bind(this)))
      }
    },

    unbind() {
      const { element, $props, cropper } = this

      if (isFunction($props.cropstart)) {
        removeListener(element, EVENT_CROP_START, $props.cropstart)
      }

      if (isFunction($props.cropmove)) {
        removeListener(element, EVENT_CROP_MOVE, $props.cropmove)
      }

      if (isFunction($props.cropend)) {
        removeListener(element, EVENT_CROP_END, $props.cropend)
      }

      if (isFunction($props.crop)) {
        removeListener(element, EVENT_CROP, $props.crop)
      }

      if (isFunction($props.zoom)) {
        removeListener(element, EVENT_ZOOM, $props.zoom)
      }

      removeListener(cropper, EVENT_POINTER_DOWN, this.onCropStart)

      if ($props.zoomable && $props.zoomOnWheel) {
        removeListener(cropper, EVENT_WHEEL, this.onWheel, {
          passive: false,
          capture: true
        })
      }

      if ($props.toggleDragModeOnDblclick) {
        removeListener(cropper, EVENT_DBLCLICK, this.onDblclick)
      }

      removeListener(element.ownerDocument, EVENT_POINTER_MOVE, this.onCropMove)
      removeListener(element.ownerDocument, EVENT_POINTER_UP, this.onCropEnd)

      if ($props.responsive) {
        removeListener(window, EVENT_RESIZE, this.onResize)
      }
    }
  }
}
